#############################################################################
##
#W    posets.gi           share package 'homology'                 Welker 
##
## Some example posets (in particular from group theory), constructors for 
## posets and their order complexes.


   
#==================================================
# Examples of Order-Relations
#==================================================


InstallGlobalFunction (IsSetPartitionRefinement, function (p1,p2)
  return IsEmpty(Filtered(p2, block2 -> not(ForAny(p1, block1 -> IsSubset(block1,block2)))));
end); 

InstallGlobalFunction (IsNumberPartitionRefinement, function (p1,p2)
  return ForAny(Arrangements(p1,Number(p1)), c -> IsCompositionRefinement (c,p2));
end);
 
InstallGlobalFunction (IsCompositionRefinement, function (c1,c2)
  local number_of_parts_c1, number_of_parts_c2, part, c2_index; 

  number_of_parts_c1 := Number(c1);
  number_of_parts_c2 := Number(c2);

  c2_index := 1;
  for part in c1 do
    while ((part > 0) and (c2_index <= number_of_parts_c2)) do
      part := part - c2[c2_index];
      c2_index := c2_index+1;
    od;
    if ((part < 0) or ((part > 0) and (c2_index > number_of_parts_c2))) then
      return false;
    fi;
  od; 
  return true; 
end);

#=======================================================
# Examples of Groundsets for the above Order Relations
#=======================================================

InstallGlobalFunction (Subsets, function (s)
  if IsEmpty(s) then
    return [s];
  else
    return Concatenation([s],Union(List(s, element -> Subsets(Difference(s,[element])))));
  fi;
end);

InstallGlobalFunction (NonTrivialSubsets, function (s)
  return Filtered(Subsets(s), set -> (Number(set) > 0) and (Number(set) < Number(s)));
end);

InstallGlobalFunction (NonTrivialPartitionsSet, function (s)
  return Filtered(PartitionsSet(s), partition -> ForAny(partition,block
    -> (Number(block) > 1) and (Number(block) < Number(s))));
end);

InstallGlobalFunction (PartitionsNumber, function (n)
  return Partitions(n);
end);

InstallGlobalFunction (NonTrivialPartitionsNumber, function (n)
  return Filtered(Partitions(n), partition -> ForAny(partition, block
    -> (block >1) and (block < n)));
end);

InstallGlobalFunction (Compositions, function(n)
  return OrderedPartitions(n);
end);

InstallGlobalFunction (NonTrivialCompositions, function (n)
  return Filtered(OrderedPartitions(n), partition -> ForAny(partition, block
    -> (block >1) and (block < n)));
end);

InstallGlobalFunction (SubSemilatticeBoolean, function(list_of_subsets)
  return DuplicateFreeList(List(
                     Combinations([1..Number(list_of_subsets)]), 
                         c -> Union(List(c, s -> list_of_subsets[s]))));
end);

 
#==================================================
# Constructors for Posets and Order Complexes
#==================================================

InstallGlobalFunction (OrderRelationToPoset, function (s,rel)
  local successors, predecessors, element, test_element, minimal,
        number_of_elements;

  successors := [];
  predecessors := [];
  number_of_elements := Number(s);
  for element in [1..number_of_elements] do
    Append(successors,[[]]);
    Append(predecessors,[[]]);
    for test_element in Difference([1..number_of_elements],[element]) do
      if rel(s[test_element],s[element]) then
        successors[element] := Concatenation(successors[element],[test_element+1]);
      else 
        if rel(s[element],s[test_element]) then
          predecessors[element] := Concatenation(predecessors[element],[test_element+1]);
        fi;
      fi;
    od;
  od;

  successors:= List(successors, element -> Filtered(element, test_element -> 
                     IsEmpty(Intersection(element,predecessors[test_element-1]))));

  minimal := [];
  for element in [1..number_of_elements] do
    if (IsEmpty(successors[element])) then
      successors[element] := [number_of_elements+2];
    fi;
    if (IsEmpty(predecessors[element])) then
      Append(minimal,[element+1]);
    fi;
   od;

   return Concatenation([minimal],Concatenation(successors,[[]]));
end);


InstallGlobalFunction (IntervalOrderComplex, function (hd,min,max) 
  local succ;
  succ := hd[min];
  if (max in succ) then
    return [[]];
  else
    return Union(List(succ, i -> List (IntervalOrderComplex(hd,i,max), c-> Concatenation ([i],c))));
  fi;
end);

InstallGlobalFunction (OrderComplex, function (hd)
  if (hd[1] = hd[2]) then
    return [];
  else
    return  IntervalOrderComplex(hd,1,Number(hd));
  fi;
end);


